--------------------------------------------------------------------------------------------
-- Broker_TSMProfile - Displays the current TSM profile and lets you switch between profiles
--------------------------------------------------------------------------------------------
-- Broker_TSMProfile.lua
--
--[[

Have you ever placed a lot of items to the auction house using a wrong Trade Skill Master profile?

Broker_TSMProfile displays the current Trade Skill Master profile and lets you
switch to a different profile with a simple click, so you will never lose gold again
because the wrong profile was set.

Broker_TSMProfile V0.1.5 has some experimental functionality to clean up TSM Accounting data 
(at character level). Useful for cleaning after deleting old alts/twinks.
 
Using a Data Broker Display is recommended but is not necessary.
Broker_TSMProfile has LDB support for Bazooka, ChocolateBar, Titan Panel, StatBlockCore etc.

Author: Caraxe
All Rights Reserved

Version 0.1.6
------------------------------------------------------------------------------------]]

-----------------------------------------------------------------------------
-- Initializing Addon

local addonName, addon = ...
addon = LibStub("AceAddon-3.0"):NewAddon(addon, addonName, "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local addonVersionString = GetAddOnMetadata(addonName, "Version")

local AceAddon = LibStub("AceAddon-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local LibQTip = LibStub("LibQTip-1.0")
local LDB = LibStub("LibDataBroker-1.1")
local Icon = LibStub("LibDBIcon-1.0")

-----------------------------------------------------------------------------
-- Encounter TSM object hiding from Ace API

local TSMName = ""
local TSMObj = nil
local TSMAccName = ""
local TSMAccObj = nil
local NewAddonOrig = nil

local function NewAddonHlp(objectorname, ...)
	local obj = NewAddonOrig(objectorname, ...)
	if obj then
		if obj.name == "TradeSkillMaster" then
			TSMName = obj.name
			TSMObj = obj
		end
		if obj.name == "TSM_Accounting" then
			TSMAccName = obj.name
			TSMAccObj = obj
		end
		if TSMObj and TSMAccObj then
			-- back to normal
			AceAddon.NewAddon = NewAddonOrig
		end
	end
	return obj
end

if AceAddon then
	-- Broker_TSMProfile needs probably update whenever a newer AceAddon version is used elsewhere
	NewAddonOrig = AceAddon.NewAddon
	AceAddon.NewAddon = NewAddonHlp
end

-----------------------------------------------------------------------------
-- LDB Integration

local function do_OnEnter(frame)
	local tooltip = LibQTip:Acquire(addon)
	tooltip:SmartAnchorTo(frame)
	tooltip:SetAutoHideDelay(0.1, frame)
	tooltip:EnableMouse(true)
	addon:UpdateTooltip()
	tooltip:Show()
end

local function do_OnClick(frame, button)
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata(addonName, "Title"))
	end
	if button == "LeftButton" then
		local editbox = ChatEdit_ChooseBoxForSend(DEFAULT_CHAT_FRAME)
		ChatEdit_ActivateChat(editbox)
		editbox:SetText("/tsm")
		ChatEdit_OnEnterPressed(editbox)
	end
	addon:UpdateTooltip()
end

addon.LDB = LDB:NewDataObject(addonName, {
	type = "data source",
	label = addonName,
	text = L["TSM Profile"] .. ": --",
	icon = "Interface\\Icons\\Trade_Engineering",
	OnEnter = do_OnEnter,
	OnClick = do_OnClick,
});

-----------------------------------------------------------------------------
-- Options

local defaults = {
	profile = {
		minimap = {
			hide = false,
		},
		experimental = {
			hide = true,
			group = true,
		},
	},
	global = {},
}

function addon:ReopenConfigDisplay(f)
	if InterfaceOptionsFrame:IsShown() then
		InterfaceOptionsFrame:Hide();
		InterfaceOptionsFrame_OpenToCategory(f)
	end
end

local nameOfRC = function (recordType)
	if recordType == "goldlog" then
		return L["Goldlog Data"]
	end
	if recordType == "auctions" then
		return L["Auctioned Item Data"]
	end
	if recordType == "buys" then
		return L["Bought Item Data"]
	end
	if recordType == "sales" then
		return L["Sold Item Data"]
	end
	if recordType == "income" then
		return L["Money Income Data"]
	end
	if recordType == "expense" then
		return L["Money Expense Data"]
	end
	if recordType == "group" then
		return L["All Character Data"]
	end
	return "--"
end

local function DeleteCharacterData(arg)
	local toon, recordType = unpack(arg)
	local cnt, items = 0, 0

	if recordType == "goldlog" then
		if TSMAccObj and TSMAccObj.goldLog and TSMAccObj.goldLog[toon] then
			-- wiping of data is possible here, but deleting of toon entry is not
			-- because of the special saving mechanism in TSM
			for i = #TSMAccObj.goldLog[toon], 1, -1 do
				cnt = cnt + 1
				tremove(TSMAccObj.goldLog[toon], i)
			end
		end
	end

	if recordType == "income" or recordType == "expense" then
		if TSMAccObj and TSMAccObj.money and TSMAccObj.money[recordType] then
			for i=#TSMAccObj.money[recordType], 1, -1 do
				if TSMAccObj.money[recordType][i].player == toon then
					cnt = cnt + 1
					tremove(TSMAccObj.money[recordType], i)
				end
			end
		end
	end

	if recordType == "auctions" or recordType == "buys" or recordType == "sales" then
		if TSMAccObj and TSMAccObj.items then
			for itemString, data in pairs(TSMAccObj.items) do
				local left = 0
				for _, rT in ipairs({"sales", "buys", "auctions"}) do
					for i=#data[rT], 1, -1 do
						if rT == recordType and data[recordType][i].player == toon then
							cnt = cnt + 1
							tremove(data[rT], i)
						end
					end
					left = left + #data[rT]
				end
				if left == 0 then
					TSMAccObj.items[itemString] = nil
					items = items + 1
				end
			end
		end
	end

	if items > 0 then
		addon:Printf(L["%s: deleted %d records and %d items for %s"], nameOfRC(recordType), cnt, items, toon)
	else
		if cnt > 0 then
			addon:Printf(L["%s: deleted %d records for %s"], nameOfRC(recordType), cnt, toon)
		end
	end
end

local function DeleteAllCharacterData(arg)
	local toon, recordType = unpack(arg)
	if recordType == "group" then
		for _, rT in ipairs({"sales", "buys", "auctions", "income", "expense", "goldlog"}) do
			DeleteCharacterData({toon, rT})
		end
	end
end

StaticPopupDialogs[addonName .. "_DELETE_group_DATA"] = {
	preferredIndex = 3,
	text = string.format(L["Are you sure you want to all character data of %s from the TSM database?"],"\n\n%s\n\n"),
	button1 = OKAY,
	button2 = CANCEL,
	OnAccept = function(self,data)
		DeleteAllCharacterData(data)
		addon:ReopenConfigDisplay(addon.optionsFrame)
		end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	enterClicksFirstButton = false,
	showAlert = true,
}

StaticPopupDialogs[addonName .. "_DELETE_goldlog_DATA"] = {
	preferredIndex = 3,
	text = string.format(L["Are you sure you want to remove all goldlog data of %s from the TSM database?"],"\n\n%s\n\n"),
	button1 = OKAY,
	button2 = CANCEL,
	OnAccept = function(self,data)
		DeleteCharacterData(data)
		addon:ReopenConfigDisplay(addon.optionsFrame)
		end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	enterClicksFirstButton = false,
	showAlert = true,
}

StaticPopupDialogs[addonName .. "_DELETE_income_DATA"] = {
	preferredIndex = 3,
	text = string.format(L["Are you sure you want to remove all money income data of %s from the TSM database?"],"\n\n%s\n\n"),
	button1 = OKAY,
	button2 = CANCEL,
	OnAccept = function(self,data)
		DeleteCharacterData(data)
		addon:ReopenConfigDisplay(addon.optionsFrame)
		end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	enterClicksFirstButton = false,
	showAlert = true,
}

StaticPopupDialogs[addonName .. "_DELETE_expense_DATA"] = {
	preferredIndex = 3,
	text = string.format(L["Are you sure you want to remove all money expense data of %s from the TSM database?"],"\n\n%s\n\n"),
	button1 = OKAY,
	button2 = CANCEL,
	OnAccept = function(self,data)
		DeleteCharacterData(data)
		addon:ReopenConfigDisplay(addon.optionsFrame)
		end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	enterClicksFirstButton = false,
	showAlert = true,
}

StaticPopupDialogs[addonName .. "_DELETE_buys_DATA"] = {
	preferredIndex = 3,
	text = string.format(L["Are you sure you want to remove all bought item data of %s from the TSM database?"],"\n\n%s\n\n"),
	button1 = OKAY,
	button2 = CANCEL,
	OnAccept = function(self,data)
		DeleteCharacterData(data)
		addon:ReopenConfigDisplay(addon.optionsFrame)
		end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	enterClicksFirstButton = false,
	showAlert = true,
}

StaticPopupDialogs[addonName .. "_DELETE_sales_DATA"] = {
	preferredIndex = 3,
	text = string.format(L["Are you sure you want to remove all sold item data of %s from the TSM database?"],"\n\n%s\n\n"),
	button1 = OKAY,
	button2 = CANCEL,
	OnAccept = function(self,data)
		DeleteCharacterData(data)
		addon:ReopenConfigDisplay(addon.optionsFrame)
		end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	enterClicksFirstButton = false,
	showAlert = true,
}

StaticPopupDialogs[addonName .. "_DELETE_auctions_DATA"] = {
	preferredIndex = 3,
	text = string.format(L["Are you sure you want to remove all auctioned item data of %s from the TSM database?"],"\n\n%s\n\n"),
	button1 = OKAY,
	button2 = CANCEL,
	OnAccept = function(self,data)
		DeleteCharacterData(data)
		addon:ReopenConfigDisplay(addon.optionsFrame)
		end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	enterClicksFirstButton = false,
	showAlert = true,
}

local function GetOptions(uiType, uiName, appName)
	if appName == addonName then
		local options = {
				type = "group",
				name = GetAddOnMetadata(addonName, "Title"),
				args = {
					desc = {
						type = "description",
						order = 0,
						name = GetAddOnMetadata(addonName, "Notes"),
					},
					general = {
						name = L["General Configuration"],
						type = "group",
						guiInline = true,
						order = 0.2,
						childGroups = "select",
						width = "double",
						args = {
							minimap = {
								name = L["Minimap Icon"],
								desc = L["Toggle minimap icon"],
								type = "toggle",
								order = 0.1,
								get = function()
									local db = addon.db.profile
									return not db.minimap.hide
									end,
								set = function()
									local db = addon.db.profile
									db.minimap.hide = not db.minimap.hide
									if db.minimap.hide then
										Icon:Hide(addonName)
									else
										Icon:Show(addonName)
									end
								end,
							},
							experimental = {
								name = L["Experimental Functions"],
								desc = L["Toggle experimental functions to delete TSM data per character and/or category"],
								type = "toggle",
								order = 0.2,
								get = function()
									local db = addon.db.profile
									return not db.experimental.hide
									end,
								set = function()
									local db = addon.db.profile
									db.experimental.hide = not db.experimental.hide
									addon:ReopenConfigDisplay(addon.optionsFrame)
								end,
							},
							group = (function()
								local ret = {}
								ret.name = L["Group per Character"]
								ret.desc = L["Show each category for a character or group data up for each character"]
								ret.type = "toggle"
								ret.order = 0.3
								ret.get = function()
										local db = addon.db.profile
										return db.experimental.group
										end
								ret.set = function()
										local db = addon.db.profile
										db.experimental.group = not db.experimental.group
										addon:ReopenConfigDisplay(addon.optionsFrame)
									end
								local db = addon.db.profile
								if db.experimental.hide then
									return nil
								else
									return ret
								end
							end)(),
						},
					},
					experimental = {
						name = L["Experimental Functions"],
						type = "group",
						guiInline = true,
						order = 0.3,
						childGroups = "select",
						width = "double",
						args = (function ()
							local ret = {}

							local db = addon.db.profile
							if db.experimental.hide then
								return ret
							end

							ret.newline0 = {
								order = 0.01,
								cmdHidden = true,
								name = L["Experimental functions to delete TSM data per character and/or category"],
								type = "description",
								width = "full",
							}

							local toons = {}
							local toonRecordCnt = {}

							if TSMAccObj and TSMAccObj.goldLog then
								local ts = "goldlog"
								toons[ts] = toons[ts] or {}
								for player, data in pairs(TSMAccObj.goldLog) do
									if data and #data > 0 then
										local tn = player
										toonRecordCnt[ts..tn] = #data
										table.insert(toons[ts],tn)
									end
								end
							end

							if TSMAccObj and TSMAccObj.money then
								if TSMAccObj.money.income then
									local ts = "income"
									toons[ts] = toons[ts] or {}
									for _, record in ipairs(TSMAccObj.money.income) do
										local tn = record.player
										if toonRecordCnt[ts..tn] then
											toonRecordCnt[ts..tn] = toonRecordCnt[ts..tn] + 1
										else
											toonRecordCnt[ts..tn] = 1
										end
										table.insert(toons[ts],tn)
									end
								end

								if TSMAccObj.money.expense then
									local ts = "expense"
									toons[ts] = toons[ts] or {}
									for _, record in ipairs(TSMAccObj.money.expense) do
										local tn = record.player
										if toonRecordCnt[ts..tn] then
											toonRecordCnt[ts..tn] = toonRecordCnt[ts..tn] + 1
										else
											toonRecordCnt[ts..tn] = 1
										end
										table.insert(toons[ts],tn)
									end
								end
							end

							if TSMAccObj and TSMAccObj.items then
								for itemString, record in pairs(TSMAccObj.items) do
									if (record.sales) then
										local ts = "sales"
										toons[ts] = toons[ts] or {}
										for _, r in pairs(record.sales) do
											local tn = r.player
											if toonRecordCnt[ts..tn] then
												toonRecordCnt[ts..tn] = toonRecordCnt[ts..tn] + 1
											else
												toonRecordCnt[ts..tn] = 1
											end
											table.insert(toons[ts],tn)
										end
									end
									if (record.buys) then
										local ts = "buys"
										toons[ts] = toons[ts] or {}
										for _, r in pairs(record.buys) do
											local tn = r.player
											if toonRecordCnt[ts..tn] then
												toonRecordCnt[ts..tn] = toonRecordCnt[ts..tn] + 1
											else
												toonRecordCnt[ts..tn] = 1
											end
											table.insert(toons[ts],tn)
										end
									end
									if (record.auctions) then
										local ts = "auctions"
										toons[ts] = toons[ts] or {}
										for _, r in pairs(record.auctions) do
											local tn = r.player
											if toonRecordCnt[ts..tn] then
												toonRecordCnt[ts..tn] = toonRecordCnt[ts..tn] + 1
											else
												toonRecordCnt[ts..tn] = 1
											end
											table.insert(toons[ts],tn)
										end
									end
								end
							end

							local deltoon = function(info)
								local toon, recordType = unpack(info.arg)
								if not toon then return end
								local dialog = StaticPopup_Show(addonName .. "_DELETE_" .. recordType .. "_DATA", toon, nil, info.arg)
							end

							ret.newline1 = {
								order = 0.40,
								cmdHidden = true,
								name = "",
								type = "description",
								width = "full",
							}

							if db.experimental.group then
								local gtoons = {}
								local ts = "group"
								gtoons[ts] = gtoons[ts] or {}
								for recordType, stoons in pairs(toons) do
									table.sort(stoons)
									for _, tn in pairs(stoons) do
										if not gtoons[tn] then
											gtoons[tn] = true
											table.insert(gtoons[ts],tn)
										end
										if toonRecordCnt[ts..tn] then
											toonRecordCnt[ts..tn] = toonRecordCnt[ts..tn] + 1
										else
											toonRecordCnt[ts..tn] = 1
										end
									end
								end
								local recordType = "group"
								local stoons = gtoons[recordType]
								ret[recordType] = {
										order = 100,
										type = "group",
										name = nameOfRC(recordType),
										guiInline = false,
										args = (function()
											local tret = {}
											table.sort(stoons)
											for ord, tn in pairs(stoons) do
												local toon = tn
												if not tret[tn.."_desc"] then
													tret[tn.."_desc"] = {
														order = function(info) return ord*10 + 0 end,
														name = toon,
														desc = tn,
														descStyle = "tooltip",
														type = "description",
														width = "normal",
														cmdHidden = true,
													}
												end
												if not tret[tn.."_cnt"] then
													tret[tn.."_cnt"] = {
														order = function(info) return ord*10 + 1 end,
														name = toonRecordCnt[recordType..tn] .. L[" records"],
														desc = tn,
														descStyle = "tooltip",
														type = "description",
														width = "normal",
														cmdHidden = true,
													}
												end
												if not tret[tn.."_delete"] then
													tret[tn.."_delete"] = {
														order = function(info) return ord*10 + 2 end,
														name = DELETE,
														desc = DELETE..":\n"..nameOfRC(recordType).." ("..tn..")",
														type = "execute",
														width = "normal",
														arg = { toon, recordType },
														func = deltoon,
													}
												end
												if not tret[tn.."_nl"] then
													tret[tn.."_nl"] = {
														order = function(info) return ord*10 + 3 end,
														name = "",
														type = "description",
														width = "full",
														cmdHidden = true,
													}
												end
											end
											return tret
											end
										)(),
								}
							else
								for recordType, stoons in pairs(toons) do
									ret[recordType] = {
										order = 100,
										type = "group",
										name = nameOfRC(recordType),
										guiInline = false,
										args = (function()
											local tret = {}
											table.sort(stoons)
											for ord, tn in pairs(stoons) do
												local toon = tn
												if not tret[tn.."_desc"] then
													tret[tn.."_desc"] = {
														order = function(info) return ord*10 + 0 end,
														name = toon,
														desc = tn,
														descStyle = "tooltip",
														type = "description",
														width = "normal",
														cmdHidden = true,
													}
												end
												if not tret[tn.."_cnt"] then
													tret[tn.."_cnt"] = {
														order = function(info) return ord*10 + 1 end,
														name = toonRecordCnt[recordType..tn] .. L[" records"],
														desc = tn,
														descStyle = "tooltip",
														type = "description",
														width = "normal",
														cmdHidden = true,
													}
												end
												if not tret[tn.."_delete"] then
													tret[tn.."_delete"] = {
														order = function(info) return ord*10 + 2 end,
														name = DELETE,
														desc = DELETE..":\n"..nameOfRC(recordType).." ("..tn..")",
														type = "execute",
														width = "normal",
														arg = { toon, recordType },
														func = deltoon,
													}
												end
												if not tret[tn.."_nl"] then
													tret[tn.."_nl"] = {
														order = function(info) return ord*10 + 3 end,
														name = "",
														type = "description",
														width = "full",
														cmdHidden = true,
													}
												end
											end
											return tret
											end
										)(),
									}
								end
							end

							return ret
							end
						)()
					},
				},
			}
		return options
	end
end

-----------------------------------------------------------------------------
-- Broker_TSMProfile

local function SecTimer()
	local profile = ""
	if TSMObj then
		profile = TSMObj.db:GetCurrentProfile()
	end
	addon.LDB.text = L["TSM Profile"] .. ": " .. profile
end

function addon:OnInitialize()
	addon.db = AceDB:New("Broker_TSMProfileDB", defaults, true)

	local db = addon.db.profile
	Icon:Register(addonName, addon.LDB, db.minimap)

	addon.externalsInfo = ""
	local TSMAddonVersion = GetAddOnMetadata("TradeSkillMaster", "Version")
	if TSMAddonVersion then
		addon.externalsInfo = "Trade Skill Master " .. TSMAddonVersion
	end

	AceConfigRegistry:RegisterOptionsTable(addonName, GetOptions)
	local optionsFrame = AceConfigDialog:AddToBlizOptions(addonName, GetAddOnMetadata(addonName, "Title"))
	addon.optionsFrame = optionsFrame
	optionsFrame.default = function()
		addon:ReopenConfigDisplay(optionsFrame)
	end
end

function addon:OnEnable()
	if TSMObj then
		-- add timer for detecting switches
		addon.sectimer = C_Timer.NewTicker(1, SecTimer)

		local profile = TSMObj.db:GetCurrentProfile()
		addon:Printf(addonVersionString .. ", " .. addon.externalsInfo .. 
			L[" detected"] .. ", " .. L["TSM Profile"] .. ": " .. profile)

		-- leave it hidden
		-- AceAddon.addons[TSMName] = TSMObj
	else
		addon:Printf(addonVersionString .. ", " .. L["no TSM object found"] .. ".")
	end
end

local function tooltip_line_OnMouseUp(frame, profile, button)
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata(addonName, "Title"))
	end
	if button == "LeftButton" then
		if TSMObj then
			TSMObj:ChangeProfile(profile)
			addon.LDB.text = L["TSM Profile"] .. ": " .. profile
		end
		addon:UpdateTooltip()
	end
end

function addon:UpdateTooltip()
	if not LibQTip:IsAcquired(addon) then
		return
	end

	local tooltip = LibQTip:Acquire(addon)
	tooltip:Clear()
	tooltip:SetColumnLayout(2,"LEFT", "RIGHT")

	local line = tooltip:AddHeader()
	tooltip:SetCell(line, 1, "|cfffed100Broker_|cffffffffTSMProfile", nil, "CENTER", 2)
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, L["Profile"])
	tooltip:SetCell(line, 2, L["State"])
	line = tooltip:AddSeparator()

	if TSMObj then
		for i, profile in ipairs(TSMObj.db:GetProfiles()) do
			line = tooltip:AddLine()
			tooltip:SetCell(line, 1, format("%d. %s", i, profile))
			if (TSMObj.db:GetCurrentProfile() == profile) then
				tooltip:SetCell(line, 2, L["On"])
			else
				tooltip:SetCell(line, 2, "--")
			end
			tooltip:SetLineScript(line, "OnMouseUp", tooltip_line_OnMouseUp, profile)
		end
	end

	line = tooltip:AddSeparator()
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "|cfffed100" .. L["Click on profile to switch"], nil, nil, 2)
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "|cfffed100" .. L["Right-Click opens options"], nil, nil, 2)

	tooltip:UpdateScrolling()
end

-- EOF
